
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Octogonal Tower prefab nodes
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootPart", "STONE_GATE_V5_OTOWER_ROOT_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1Part", "STONE_GATE_V5_OTOWER_TILING1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2Part", "STONE_GATE_V5_OTOWER_TILING2_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3Part", "STONE_GATE_V5_OTOWER_TILING3_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_3_Octogonal_TopPart", "STONE_GATE_V5_OTOWER_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
	"STONE_GATE_V5_OTOWER_TOP",
	"STONE_GATE_V5_OTOWER_TILING1",
	"STONE_GATE_V5_OTOWER_TILING2",
	"STONE_GATE_V5_OTOWER_TILING3"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the stone gate set main root part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_OTOWER_ROOT_PART",
	Name = "STONE_GATE_V5_OTOWER_ROOT_NAME",
	Description = "STONE_GATE_V5_OTOWER_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "STONE_GATE_V5_OTOWER_ROOT_PREFAB",
		EndPart = "STONE_GATE_V5_OTOWER_TOP_PART",
		FillerList = {
			"STONE_GATE_V5_OTOWER_TILING2_PART",
			"STONE_GATE_V5_OTOWER_TILING3_PART"
                },
		BasementFillerList = {
			"STONE_GATE_V5_OTOWER_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 12,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_3_Octogonal_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootPart", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4.6,
        SoftRadius = 4.6,
--        Strength = 10,
        IsPermanent = true
})

-- Override building part components properties (generated by the building asset processor) to add building zones
StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 9, 9 }
})

function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/" .. _nameFbx .. "Part", "STONE_GATE_V5_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_" .. _nameAsset .. "_PART",
	Name = "STONE_GATE_V5_" .. _nameAsset .. "_NAME",
	Description = "STONE_GATE_V5_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "STONE_GATE_V5_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 8, 8 }
    })
end

registerRoofPart( "OctogonalRoof", "OTOWER_ROOF" )

---- Ramparts...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/ramparts/rampartOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/ramparts/rampartOctogonalTower.fbx/Prefab/octogonalTower/step_4_Octogonal_" .. _nameFbx .. "Part", "STONE_GATE_V5_OTOWER_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_OTOWER_" .. _nameAsset .. "_PART",
	Name = "STONE_GATE_V5_OTOWER_" .. _nameAsset .. "_NAME",
	Description = "STONE_GATE_V5_OTOWER_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "STONE_GATE_V5_OTOWER_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/ramparts/rampartOctogonalTower.fbx/Prefab/octogonalTower/step_4_Octogonal_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 4, 4 }
    })
end

registerRampartPart( "Rampart", "RAMPART" )
